//  Beispiel 6.6.5 "Meine erste Webseite"
//  
//  Unter Verwendung der IoT32 Brick Library

#include <all_IoT32.h>

#include <DHT.h>

#define DHT_TYPE DHT11                                              // Sensortyp definieren für DHT11 
const int DHT_PIN = 14;                                             // Datenleitung des Sensors an GPIO14 des IoT Bricks
String Temperature;
String Humidity;

DHT dht(DHT_PIN, DHT_TYPE);                                         // Variable vom Typ DHT definieren

int counter = 0;

void setup()
{
  IoT_Init(true);
  IoT_WLANconnect(WiFiHotSpotName, WiFiHotSpotPassword);
  IoT_NTPinit();
  dht.begin();                                                      // Sensor initialisieren
                                                                    // Sobald der Browser direkt auf das Stammverzeichnis zugreift, 
  IoT_WebServer.on("/", handleRoot);                                // führe die Funktion handleRoot (siehe unten) aus.
  IoT_WebServer.begin();                                            // ab jetzt "hört" der Server auf HTTP-Anfragen
  Serial.println("HTTP server started");
}

void loop()
{
  IoT_Idle();

  IoT_WebServer.handleClient();                                     // Bediene die http Anfragen
  
  if (IoT_Keypress())                                               // Wenn Taster gedrückt wird, Konfiguration zeigen
  {
    IoT_DisplayClear(10);                                           // OLED Display löschen (10 Punkt Schrift)
    String state = IoT_WLANstatus();                                // Verbindungsstatus
    if (IoT_WLANinited)                                             // Falls WLAN Verbindung erfolgreich, Status & IP im OLED Display anzeigen
    {
      IoT_DisplayDrawText(5, 0, "WLAN: " + IoT_WLANssid());         // WLAN Netzwerk Name
      if (state == "Connected")                                     // Wenn erfolgreich verbunden, dann Signalstärke hinzufügen
      {
        state = state + " (" + str(IoT_WLANrssi()) + "dBm)";        // Signalstärke
      }
      IoT_DisplayDrawText(5, 10, state);                            // Verbindungsstatus & Signalstärke
      IoT_DisplayDrawText(5, 20, "IP: " + IoT_WLANaddress(true));   // IP Adresse
      IoT_DisplayDrawText(5, 30, "GW: " + IoT_WLANgateway(true));   // IP Gateway
      IoT_DisplayDrawText(5, 40, "NT: " + IoT_WLANsubnet(true));    // IP Subnetzmaske
    }
    else                                                            // Falls WLAN Verbindung gescheitert ist, Status im OLED Display anzeigen
    {
      IoT_DisplayDrawText(5, 10, state); 
    }
  }
  else
  {
    IoT_Idle();
    if (counter % 25 == 0) //Infos seriell nicht zu oft ausgeben (ca. alle 2 Sekunden)
    {
      if (IoT_NTPeventAvail)                                        // Zeitevent triggern
      {
        IoT_NTPprintEvent(IoT_NTPcurrentEvent);
        IoT_NTPeventAvail = false;
      }
      Serial.println(IoT_NTPdatetime() + " (" + IoT_NTPdaylightSavingTime(true) + ")");
      Serial.print("WiFi is ");
      Serial.print(IoT_WLANconnected() ? "connected" : "not connected");
      Serial.println(". Uptime: " + IoT_WLANuptime(true) + " seit " + IoT_WLANstartDatetime());
    }
    
    IoT_DisplayClear(16);                                           // OLED Display löschen (16 Punkt Schrift)
    IoT_DisplayAlignText(TEXT_ALIGN_CENTER);                        // Zentrierte Darstellung des Textes
    if (IoT_NTPvalid())                                             // Prüfen, ob eine gültige Uhrzeit aus dem Internet empfangen wurde
    {
      IoT_DisplayDrawText(63, 0, IoT_NTPtime());
      IoT_DisplayDrawText(63, 15, IoT_NTPdate());
      IoT_DisplayAlignText(TEXT_ALIGN_LEFT);                        // Linksbündige Darstellung des Textes
      if (counter % 100 == 0)                                       // Sensor etwa alle 10 Sekunden abfragen
      {
        Serial.print("Reading Sensors... ");
        IoT_Idle();
        float t = dht.readTemperature();                            //Temperatur auslesen (Celsius)
        if (not(isnan(t)))
        {
          Temperature = strrealform (t, 32, 1, 2, true, false) + " °C"; // 2 Nachkommastellen, mit Tausenderpunkt, mit ggf. Nullen nach dem Komma
          Serial.print(Temperature + ". ");
        }
        else
        {
          Serial.print("No Temperature Sensor. ");
        }
        IoT_Idle();
        float h = dht.readHumidity();                               //Feuchtigkeit auslesen (Prozent)
        if (not(isnan(h)))
        {
          Humidity = strrealform (h, 32, 1, 2, true, false) + " %"; // 2 Nachkommastellen, mit Tausenderpunkt, mit ggf. Nullen nach dem Komma
          Serial.print(Humidity + ". ");
        }
        else
        {
          Serial.print("No Humidity Sensor. ");
        }
        Serial.println("");
      }
      IoT_Idle();
    }
    else                                                            // "Bitte warten..." zentriert anzeigen
    {
      IoT_DisplayClear(24);                                         // OLED Display löschen (24 Punkt Schrift)
      IoT_DisplayAlignText(TEXT_ALIGN_CENTER);                      // Zentrierte Darstellung des Textes
      IoT_DisplayDrawText(64, 5, "Bitte");
      IoT_DisplayDrawText(64, 34, "warten...");
    }
    if (Temperature != "")
    {
      IoT_DisplayDrawText(5, 30, "Temp: " + Temperature);          // Temperatur anzeigen
    }
    else
    {
      IoT_DisplayDrawText(5, 30, "Temp: (fehlt)");                   // Temperatur-Sensor nicht gefunden
    }
    if (Humidity != "")
    {
      IoT_DisplayDrawText(5, 45, "Feuch: " + Humidity);             // Feuchtigkeit anzeigen
    }
    else
    {
      IoT_DisplayDrawText(5, 45, "Feuch: (fehlt)");                   // Feuchtigkeits-Sensor nicht gefunden
    }
  }
  IoT_DisplayUpdate();                                              // OLED-Anzeige aktualisieren
  delay(100);                                                       // 100 ms. warten
  counter++;
}

//Mit der Funktion handleRoot() wird die Website ausgeliefert sobald eine Anfrage von einem Browser eintrifft

void handleRoot()
{
  String content;
  String time_web = IoT_NTPtime();
  String date_web = IoT_NTPdate();
  String temp_web = Temperature;
  String humi_web = Humidity;
  content = "<!DOCTYPE html>";
  content += "<html>";
  content += "<head>";
  // Falls du die Werte auf deiner Website aktualisieren möchtest kannst du folgende Zeile verwenden. 
  // Vorher musst du die IP-Adresse durch die von deinem IoT Brick ersetzen:
  // content += "<meta http-equiv=\"refresh\" content=\"5; URL=http://192.168.1.153\">"; 
  // zur automatischen Aktualisierung der Seite nach 5 Sekunden
  content += "<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\">";  // wichtig, damit das Grad-Zeichen "°" korrekt dargestellt wird
  content += "<title>Meine erste IoT Brick-Website</title>";
  content += "</head>";
  content += "<body>";
  content += "<h1> Hello World! </h1>";
  content += "<p>Das ist eine sehr einfache Website von deinem IoT Brick, die Datum, Uhrzeit, Temperatur und Feuchtigkeit anzeigt.</p>";
  content += "<h2>Datum: "+date_web+"</h2>";
  content += "<h2>Uhrzeit: "+time_web+"</h2>";
  content += "<h2>Temperatur: "+temp_web+"</h2>";
  content += "<h2>Feuchte: "+humi_web+"</h2>";
  content += "<p>Durch Neuladen der Website können die Werte jederzeit aktualisiert werden.</p>";
  content += "</body>";
  content += "</html>";
 IoT_WebServer.send(200, "text/html", content);   // HTTP-Statuscode 200 = OK (Anfrage wurde erfolgreich bearbeitet)
}

